﻿using System;
using Humanizer;
namespace FoxHumanizer
{
    /// <summary>
    /// Helper to create humanized words of numbers dates and other occasions
    ///
    /// Wrapper around the Humanizer library:
    /// https://github.com/Humanizr/Humanizer
    /// </summary>
    public class FoxHumanizer
    {
        /// <summary>
        /// Humanizes a local date as yesterday, two days ago, a year ago, next month etc.
        /// </summary>
        /// <param name="date">
        /// The local date.
        /// </param>
        /// <returns>
        /// The humanized date.
        /// </returns>
        public string HumanizeDate(DateTime date)
        {
            return date.Humanize(false);
        }

        /// <summary>
        /// Humanizes a local date using the specified culture.
        /// </summary>
        /// <param name="date">
        /// The local date.
        /// </param>
        /// <param name="culture">
        /// The culture to use.
        /// </param>
        /// <returns>
        /// The humanized date.
        /// </returns>
        public string HumanizeDateForCulture(DateTime date, System.Globalization.CultureInfo culture)
        {
            return date.Humanize(false, null, culture);
        }

        /// <summary>
        /// Humanizes a string, handling things like underscores and Pascal casing.
        /// </summary>
        /// <param name="text">
        /// The string to humanize.
        /// </param>
        /// <returns>
        /// The humanized string.
        /// </returns>
        public string Humanize(string text)
        {
            return text.Humanize();
        }

        /// <summary>
        /// Turns integer numbers into words.
        /// </summary>
        /// <param name="number">
        /// The number.
        /// </param>
        /// <returns>
        /// The humanized number.
        /// </returns>
        public string NumberToWords(int number)
        {
            return number.ToWords();
        }

        /// <summary>
        /// Returns a number like 1st, 2nd, 3rd.
        /// </summary>
        /// <param name="number">
        /// The number.
        /// </param>
        /// <returns>
        /// The humanized number.
        /// </returns>
        public string NumberToOrdinal(int number)
        {
            return number.Ordinalize();
        }

        /// <summary>
        /// Returns a number like first, second, third.
        /// </summary>
        /// <param name="number">
        /// The number.
        /// </param>
        /// <returns>
        /// The humanized number.
        /// </returns>
        public string NumberToOrdinalWords(int number)
        {
            return number.ToOrdinalWords();
        }

        /// <summary>
        /// Returns a string like one car or two bananas from a qty and a string that is pluralized as needed.
        /// </summary>
        /// <param name="single">
        /// The word as its single representation.
        /// </param>
        /// <param name="qty">
        /// The quantity.
        /// </param>
        /// <returns>
        /// The humanized expression.
        /// </returns>
        public string ToQuantity(string single, int qty)
        {
            return single.ToQuantity(qty, ShowQuantityAs.Words);
        }

        /// <summary>
        /// Takes a singular noun and pluralizes it.
        /// </summary>
        /// <param name="single">
        /// The word as its single representation.
        /// </param>
        /// <returns>
        /// The plural version of the word.
        /// </returns>
        public string Pluralize(string single)
        {
            return single.Pluralize(true);
        }

        /// <summary>
        /// Takes a pluralized noun and turns it to singular.
        /// </summary>
        /// <param name="pluralized">
        /// The word as its single representation.
        /// </param>
        /// <returns>
        /// The singular version of the word.
        /// </returns>
        public string Singularize(string pluralized)
        {
            return pluralized.Singularize(true);
        }

        /// <summary>
        /// Returns a byte count as KB, MB, etc. to two places.
        /// </summary>
        /// <param name="byteSize">
        /// The number of bytes.
        /// </param>
        /// <returns>
        /// The humanized expression.
        /// </returns>
        public string ToByteSize(int byteSize)
        {
            return byteSize.Bytes().Humanize("#.##");
        }
    }
}