﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Mail;
using System.Web.Script.Serialization;

namespace SMS
{
    /// <summary>
    /// Adapted from http://csharphelper.com/blog/2018/01/how-to-send-an-sms-text-message-in-c/
    /// </summary>
    public class SMSLibrary
    {
        /// <summary>
        /// The message of any error that occurs.
        /// </summary>
        public string ErrorMessage { get; set; } = "";

        /// <summary>
        /// The name of the sender.
        /// </summary>
        public string SenderName { get; set; }

        /// <summary>
        /// The email address of the sender.
        /// </summary>
        public string SenderEmail { get; set; }

        /// <summary>
        /// The password to connect to the mail server.
        /// </summary>
        public string Password { get; set; }

        /// <summary>
        /// The mail server.
        /// </summary>
        public string Server { get; set; }

        /// <summary>
        /// The port for the mail server.
        /// </summary>
        public int Port { get; set; } = 25;

        /// <summary>
        /// True to use SSL to connect to the mail server.
        /// </summary>
        public bool UseSSL { get; set; } = false;

        /// <summary>
        /// An array of valid countries.
        /// </summary>
        public string[] Countries
        {
            get
            {
                if (_countryInfos.Count == 0)
                {
                    GetInfo();
                }
                return _countryInfos.Select(c => c.Value.CountryName).ToArray();
            }
        }

        /// <summary>
        /// A set of CountryInfo objects for valid countries.
        /// </summary>
        private Dictionary<string, CountryInfo> _countryInfos = new Dictionary<string, CountryInfo>();

        /// <summary>
        /// Returns a list of carriers for the specified country.
        /// </summary>
        /// <param name="country">
        /// The country to get carriers for.
        /// </param>
        /// <returns>
        /// An array of carriers for the specified country or a blank array for a non-existent country.
        /// </returns>
        public string[] GetCarriersForCountry(string country)
        {
            string[] carriers;
            try
            {
                CountryInfo countryInfo = _countryInfos[country];
                carriers = countryInfo.Carriers.Select(c => c.Key).ToArray();
            }
            catch (Exception)
            {
                ErrorMessage = "Country not found";
                carriers = new string[0];
            }
            return carriers;
        }

        /// <summary>
        /// Returns a list of emails for the specified carrier in the specified country.
        /// </summary>
        /// <param name="country">
        /// The country the carriers belongs to.
        /// </param>
        /// <param name="carrier">
        /// The carrier to get emails for.
        /// </param>
        /// <returns>
        /// An array of emails for the specified carrier in the specified country or a blank array for a non-existent country or carrier.
        /// </returns>
        public string[] GetEmailsForCarrier(string country, string carrier)
        {
            string[] emails;
            try
            {
                CountryInfo countryInfo = _countryInfos[country];
                try
                {
                    CarrierInfo carrierInfo = countryInfo.Carriers[carrier];
                    emails = carrierInfo.Emails.ToArray();
                }
                catch (Exception)
                {
                    ErrorMessage = "Carrier not found";
                    emails = new string[0];
                }
            }
            catch (Exception)
            {
                ErrorMessage = "Country not found";
                emails = new string[0];
            }
            return emails;
        }

        /// <summary>
        /// Send an SMS message.
        /// </summary>
        /// <param name="name">
        /// The recipient's name.
        /// </param>
        /// <param name="phone">
        /// The recipient's phone number.
        /// </param>
        /// <param name="subject"></param>
        /// <param name="body"></param>
        public void SendMessage(string name, string phone, string carrierEmail, string subject, string body)
        {
            // Strip unwanted characters from the phone number and create the email address to use.
            string toPhone = phone.Trim().Replace("-", "").Replace("(", "").Replace(")", "").Replace("+", "");
            string toEmail = toPhone + "@" + carrierEmail;

            // Create the mail message.
            MailAddress from_address = new MailAddress(SenderEmail, SenderName);
            MailAddress to_address = new MailAddress(toEmail, name);
            MailMessage message = new MailMessage(from_address, to_address);
            message.Subject = subject;
            message.Body = body;

            // Get the SMTP client.
            SmtpClient client = new SmtpClient()
            {
                Host = Server,
                Port = Port,
                EnableSsl = UseSSL,
                UseDefaultCredentials = false,
                Credentials = new NetworkCredential(from_address.Address, Password),
            };

            // Send the message.
            client.Send(message);
        }

        /// <summary>
        /// Gets the SMS information for the library.
        /// </summary>
        private void GetInfo()
        {
            // Get the data file.
            const string url = "https://raw.github.com/cubiclesoft/email_sms_mms_gateways/master/sms_mms_gateways.txt";
            string serialization = GetTextFile(url);

            // Deserialize it.
            JavaScriptSerializer serializer = new JavaScriptSerializer();
            Dictionary<string, object> dict = (Dictionary<string, object>)serializer.DeserializeObject(serialization);

            // Get the countries.
            Dictionary<string, object> countries = (Dictionary<string, object>)dict["countries"];
            Dictionary<string, CountryInfo> countriesByAbbrev = new Dictionary<string, CountryInfo>();
            foreach (KeyValuePair<string, object> pair in countries)
            {
                CountryInfo country_info = new CountryInfo() { CountryAbbreviation = pair.Key, CountryName = (string)pair.Value };
                _countryInfos.Add(country_info.CountryName, country_info);
                countriesByAbbrev.Add(country_info.CountryAbbreviation, country_info);
            }

            // Get the SMS carriers.
            Dictionary<string, object> sms_carriers = (Dictionary<string, object>)dict["sms_carriers"];
            foreach (KeyValuePair<string, object> pair in sms_carriers)
            {
                // Get the corresponding CountryInfo.
                CountryInfo country_info = countriesByAbbrev[pair.Key];

                // Get the country's carriers.
                Dictionary<string, object> carriers = (Dictionary<string, object>)pair.Value;
                foreach (KeyValuePair<string, object> carrier_pair in carriers)
                {
                    // Create a CarrierInfo for this carrier.
                    CarrierInfo carrier_info = new CarrierInfo() { CarrierAbbreviation = carrier_pair.Key };
                    object[] carrier_values = (object[])carrier_pair.Value;
                    carrier_info.CarrierName = (string)carrier_values[0];
                    for (int email_index = 1; email_index < carrier_values.Length; email_index++)
                    {
                        string email = (string)carrier_values[email_index];
                        carrier_info.Emails.Add(email.Replace("{number}@", ""));
                    }
                    country_info.Carriers.Add(carrier_info.CarrierName, carrier_info);
                }
            }
        }

        /// <summary>
        /// Get the text file at the specified URL.
        /// </summary>
        /// <param name="url">
        /// The URL for the file to retrieve.
        /// </param>
        /// <returns>
        /// The content of the file or an empty string if it failed.
        /// </returns>
        private string GetTextFile(string url)
        {
            string result = "";
            try
            {
                url = url.Trim();
                if (!url.ToLower().StartsWith("http"))
                {
                    url = "http://" + url;
                }
                WebClient web_client = new WebClient();
                MemoryStream image_stream = new MemoryStream(web_client.DownloadData(url));
                StreamReader reader = new StreamReader(image_stream);
                result = reader.ReadToEnd();
                reader.Close();
            }
            catch (Exception ex)
            {
                ErrorMessage = "Error downloading file " + url + '\n' + ex.Message;
            }
            return result;
        }
    }
}
