﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Mail;

namespace SMTPLibrary
{
    /// <summary>
    /// Sends email via SMTP.
    /// </summary>
    public class SMTP
    {
        /// <summary>
        /// The mail server address.
        /// </summary>
        public string MailServer { get; set; } = "";

        /// <summary>
        /// The port to use.
        /// </summary>
        public int ServerPort { get; set; } = 25;

        /// <summary>
        /// The email address for the sender.
        /// </summary>
        public string SenderEmail { get; set; } = "";

        /// <summary>
        /// The name of the sender.
        /// </summary>
        public string SenderName { get; set; } = "";

        /// <summary>
        /// A semi-colon delimited list of recipients.
        /// </summary>
        public string Recipients { get; set; } = "";

        /// <summary>
        /// A semi-colon delimited list of CC recipients.
        /// </summary>
        public string CCRecipients { get; set; } = "";

        /// <summary>
        /// A semi-colon delimited list of BCC recipients.
        /// </summary>
        public string BCCRecipients { get; set; } = "";

        /// <summary>
        /// The subject.
        /// </summary>
        public string Subject { get; set; } = "";

        /// <summary>
        /// The body of the message.
        /// </summary>
        public string Message { get; set; } = "";

        /// <summary>
        /// The user name for the server.
        /// </summary>
        public string UserName { get; set; } = "";

        /// <summary>
        /// The password for the server.
        /// </summary>
        public string Password { get; set; } = "";

        /// <summary>
        /// True if the body contains HTML.
        /// </summary>
        public bool UseHtml { get; set; }

        /// <summary>
        /// True to use SSL.
        /// </summary>
        public bool UseSsl { get; set; }

        /// <summary>
        /// The timeout in seconds.
        /// </summary>
        public int Timeout { get; set; } = 30;

        /// <summary>
        /// The text of any error that occurred.
        /// </summary>
        public string ErrorMessage { get; private set; } = "";

        /// <summary>
        /// A list of attachments.
        /// </summary>
        private List<string> _attachments = new List<string>();

        /// <summary>
        /// Send the message.
        /// </summary>
        /// <returns>
        /// True if it succeeds.
        /// </returns>
        public bool SendMail()
        {
            // Set up the host.
            NetworkCredential basicCredential = new NetworkCredential(UserName, Password);
            SmtpClient smtpClient = new SmtpClient();
            smtpClient.Host = MailServer;
            smtpClient.UseDefaultCredentials = false;
            smtpClient.Credentials = basicCredential;
            smtpClient.Timeout = Timeout * 1000;
            smtpClient.Port = ServerPort;
            smtpClient.EnableSsl = UseSsl;

            // Set up the mail message.
            MailMessage message = new MailMessage();
            message.From = new MailAddress(SenderEmail, SenderName);
            message.Sender = message.From;
            message.IsBodyHtml = UseHtml;
            message.Body = Message;
            message.Subject = Subject;

            // Handle addresses.
            foreach (var address in Recipients.Split(new[] { ";" }, StringSplitOptions.RemoveEmptyEntries))
            {
                message.To.Add(address);
            }
            foreach (var address in CCRecipients.Split(new[] { ";" }, StringSplitOptions.RemoveEmptyEntries))
            {
                message.CC.Add(address);
            }
            foreach (var address in BCCRecipients.Split(new[] { ";" }, StringSplitOptions.RemoveEmptyEntries))
            {
                message.Bcc.Add(address);
            }

            // Handle attachments.
            foreach (string attachment in _attachments)
            {
                message.Attachments.Add(new Attachment(attachment));
            }

            // Try to send the message.
            bool result = false;
            ErrorMessage = "";
            try
            {
                smtpClient.Send(message);
                result = true;
            }
            catch (Exception ex)
            {
                ErrorMessage = ex.Message;
            }
            message.Dispose();
            return result;
        }

        /// <summary>
        /// Add an attachment.
        /// </summary>
        /// <param name="fileName">
        /// The file name of the attachment.
        /// </param>
        public void AddAttachment(string fileName)
        {
            _attachments.Add(fileName);
        }
    }
}
